@@ -183,6 +183,68 @@ class _ArchetypeField(SonolusDescriptor):
             target._copy_from_(value)
 
 
+class _NameDescriptor(SonolusDescriptor):
+    def __init__(self, name: str):
+        self.name = name
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self.name
+        elif ctx():
+            raise RuntimeError("Cannot access archetype name in from self in a callback, use ArchetypeClass.name")
+        else:
+            return self.name
+
+    def __set__(self, instance, value):
+        raise AttributeError("Archetype name is read-only and cannot be set")
+
+
+class _IsScoredDescriptor(SonolusDescriptor):
+    def __init__(self, value: bool):
+        self.value = value
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self.value
+        elif ctx():
+            raise RuntimeError("Cannot access is_scored from self in a callback, use ArchetypeClass.is_scored")
+        else:
+            return self.value
+
+    def __set__(self, instance, value):
+        raise AttributeError("is_scored is read-only and cannot be set")
+
+
+class _IdDescriptor(SonolusDescriptor):
+    def __get__(self, instance, owner):
+        if not ctx():
+            raise RuntimeError("Archetype id is only available during compilation")
+        if instance is None:
+            result = ctx().global_state.archetypes.get(owner)
+            if result is None:
+                raise RuntimeError("Archetype is not registered")
+            return result
+        else:
+            return instance._info.archetype_id
+
+    def __set__(self, instance, value):
+        raise AttributeError("Archetype id is read-only and cannot be set")
+
+
+class _KeyDescriptor(SonolusDescriptor):
+    def __init__(self, value: int | float):
+        self.value = value
+
+    def __get__(self, instance, owner):
+        if instance is not None and ctx():
+            return ctx().global_state.keys[instance.id]
+        else:
+            return self.value
+
+    def __set__(self, instance, value):
+        raise AttributeError("Archetype key is read-only and cannot be set")
+
+
 def imported(*, name: str | None = None) -> Any:
     """Declare a field as imported.
 
@@ -190,7 +252,8 @@ def imported(*, name: str | None = None) -> Any:
 
     In watch mode, data may also be loaded from a corresponding exported field in play mode.
 
-    Imported fields may only be updated in the `preprocess` callback, and are read-only in other callbacks.
+    Imported fields may only be updated in the [`preprocess`][sonolus.script.archetype.PlayArchetype.preprocess]
+    callback, and are read-only in other callbacks.
 
     Usage:
         ```python
@@ -205,10 +268,12 @@ def imported(*, name: str | None = None) -> Any:
 def entity_data() -> Any:
     """Declare a field as entity data.
 
-    Entity data is accessible from other entities, but may only be updated in the `preprocess` callback
+    Entity data is accessible from other entities, but may only be updated in the
+    [`preprocess`][sonolus.script.archetype.PlayArchetype.preprocess] callback
     and is read-only in other callbacks.
 
-    It functions like `imported` and shares the same underlying storage, except that it is not loaded from a level.
+    It functions like [`imported`][sonolus.script.archetype.imported] and shares the same underlying storage,
+    except that it is not loaded from a level.
 
     Usage:
         ```python
@@ -242,7 +307,8 @@ def entity_memory() -> Any:
     Entity memory is private to the entity and is not accessible from other entities. It may be read or updated in any
     callback associated with the entity.
 
-    Entity memory fields may also be set when an entity is spawned using the `spawn()` method.
+    Entity memory fields may also be set when an entity is spawned using the
+    [`spawn()`][sonolus.script.archetype.PlayArchetype.spawn] method.
 
     Usage:
         ```python
@@ -260,7 +326,9 @@ def shared_memory() -> Any:
     Shared memory is accessible from other entities.
 
     Shared memory may be read in any callback, but may only be updated by sequential callbacks
-    (`preprocess`, `update_sequential`, and `touch`).
+    ([`preprocess`][sonolus.script.archetype.PlayArchetype.preprocess],
+    [`update_sequential`][sonolus.script.archetype.PlayArchetype.update_sequential],
+    and [`touch`][sonolus.script.archetype.PlayArchetype.touch]).
 
     Usage:
         ```python
@@ -381,6 +449,28 @@ class _BaseArchetype:
 
     _data_: _ArchetypeData
 
+    id: int = 0
+    """The id of the archetype or entity.
+
+    If accessed on an entity, always returns the runtime archetype id of the entity, even if it doesn't match the type
+    that was used to access it.
+
+    E.g. if an entity of archetype `A` is accessed via [`EntityRef[B]`][sonolus.script.archetype.EntityRef], the id will
+    still be the id of `A`.
+    """
+
+    key: int | float = -1
+    """An optional key for the archetype.
+
+    May be useful to identify an archetype in an inheritance hierarchy without needing to check id.
+
+    If accessed on an entity, always returns the runtime key of the entity, even if it doesn't match the type
+    that was used to access it.
+
+    E.g. if an entity of archetype `A` is accessed via [`EntityRef[B]`][sonolus.script.archetype.EntityRef], the key
+    will still be the key of `A`.
+    """
+
     name: ClassVar[str | None] = None
     """The name of the archetype.
 
@@ -427,17 +517,7 @@ class _BaseArchetype:
     def is_at(cls, index: int) -> bool:
         if not ctx():
             raise RuntimeError("is_at is only available during compilation")
-        return entity_info_at(index).archetype_id == cls.id()
-
-    @classmethod
-    @meta_fn
-    def id(cls) -> int:
-        if not ctx():
-            raise RuntimeError("Archetype id is only available during compilation")
-        result = ctx().global_state.archetypes.get(cls)
-        if result is None:
-            raise RuntimeError("Archetype is not registered")
-        return result
+        return entity_info_at(index).archetype_id == cls.id
 
     @classmethod
     @meta_fn
@@ -459,7 +539,7 @@ class _BaseArchetype:
         cls._init_fields()
         if not ctx():
             raise RuntimeError("Spawn is only allowed within a callback")
-        archetype_id = cls.id()
+        archetype_id = cls.id
         bound = cls._spawn_signature_.bind_partial(**kwargs)
         bound.apply_defaults()
         data = []
@@ -491,7 +571,7 @@ class _BaseArchetype:
         return entries
 
     def __init_subclass__(cls, **kwargs):
-        if cls.__module__ == _BaseArchetype.__module__:
+        if cls.__module__ == _BaseArchetype.__module__ and not getattr(cls, "_is_derived_", False):
             if cls._supported_callbacks_ is None:
                 raise TypeError("Cannot directly subclass Archetype, use the Archetype subclass for your mode")
             cls._default_callbacks_ = {getattr(cls, cb_info.py_name) for cb_info in cls._supported_callbacks_.values()}
@@ -505,6 +585,11 @@ class _BaseArchetype:
                 continue
             cls._callbacks_.append(cb)
         cls._field_init_done = False
+        cls.id = _IdDescriptor()
+        cls._key_ = cls.key
+        cls.key = _KeyDescriptor(cls.key)
+        cls.name = _NameDescriptor(cls.name)
+        cls.is_scored = _IsScoredDescriptor(cls.is_scored)
 
     @classmethod
     def _init_fields(cls):
@@ -515,7 +600,19 @@ class _BaseArchetype:
             if hasattr(mro_entry, "_field_init_done"):
                 mro_entry._init_fields()
         field_specifiers = get_field_specifiers(
-            cls, skip={"name", "is_scored", "_callbacks_", "_field_init_done"}
+            cls,
+            skip={
+                "id",
+                "key",
+                "name",
+                "is_scored",
+                "_key_",
+                "_derived_base_",
+                "_is_derived_",
+                "_default_callbacks_",
+                "_callbacks_",
+                "_field_init_done",
+            },
         ).items()
         if not hasattr(cls, "_imported_fields_"):
             cls._imported_fields_ = {}
@@ -650,6 +747,41 @@ class _BaseArchetype:
     def _post_init_fields(cls):
         pass
 
+    @classmethod
+    def derive[T](cls: type[T], name: str, is_scored: bool, key: int | float | None = None) -> type[T]:
+        """Derive a new archetype class from this archetype.
+
+        Roughly equivalent to returning:
+        ```python
+        class Derived(cls):
+            name = <name>
+            is_scored = <is_scored>
+            key = <key>  # Only set if key is not None
+        ```
+
+        This is used to create a new archetype with the same fields and callbacks, but with a different name and
+        whether it is scored. Compared to manually subclassing, this method also enables faster compilation when
+        the same base archetype has multiple derived archetypes by compiling callbacks only once for the base archetype.
+
+        Args:
+            name: The name of the new archetype.
+            is_scored: Whether the new archetype is scored.
+            key: A key that can be accessed via the `key` property of the new archetype.
+
+        Returns:
+            A new archetype class with the same fields and callbacks as this archetype, but with the given name and
+            whether it is scored.
+        """
+        if getattr(cls, "_is_derived_", False):
+            raise RuntimeError("Cannot derive from a derived archetype")
+        cls_dict = {"name": name, "is_scored": is_scored, "_is_derived_": True, "_derived_base_": cls}
+        if key is not None:
+            if not isinstance(key, (int, float)):
+                raise TypeError(f"Key must be an int or float, got {type(key)}")
+            cls_dict["key"] = key
+        new_cls = type(name, (cls,), cls_dict)
+        return new_cls
+
 
 class PlayArchetype(_BaseArchetype):
     """Base class for play mode archetypes.
@@ -688,7 +820,7 @@ class PlayArchetype(_BaseArchetype):
     def spawn_order(self) -> float:
         """Return the spawn order of the entity.
 
-        Runs when the level is loaded after `preprocess`.
+        Runs when the level is loaded after [`preprocess`][sonolus.script.archetype.PlayArchetype.preprocess].
         """
 
     def should_spawn(self) -> bool:
@@ -709,13 +841,15 @@ class PlayArchetype(_BaseArchetype):
         Runs first each frame.
 
         This is where logic affecting shared memory should be placed.
-        Other logic should be placed in `update_parallel` for better performance.
+        Other logic should typically be placed in
+        [`update_parallel`][sonolus.script.archetype.PlayArchetype.update_parallel]
+        for better performance.
         """
 
     def update_parallel(self):
         """Perform parallel actions for this frame.
 
-        Runs after `touch` each frame.
+        Runs after [`touch`][sonolus.script.archetype.PlayArchetype.touch] each frame.
 
         This is where most gameplay logic should be placed.
         """
@@ -723,7 +857,7 @@ class PlayArchetype(_BaseArchetype):
     def touch(self):
         """Handle user input.
 
-        Runs after `update_sequential` each frame.
+        Runs after [`update_sequential`][sonolus.script.archetype.PlayArchetype.update_sequential] each frame.
         """
 
     def terminate(self):
@@ -799,7 +933,7 @@ class PlayArchetype(_BaseArchetype):
             raise RuntimeError("Calling life is only allowed within a callback")
         match self._data_:
             case _ArchetypeSelfData() | _ArchetypeReferenceData():
-                return _deref(ctx().blocks.ArchetypeLife, self.id() * ArchetypeLife._size_(), ArchetypeLife)
+                return _deref(ctx().blocks.ArchetypeLife, self.id * ArchetypeLife._size_(), ArchetypeLife)
             case _:
                 raise RuntimeError("Life is not available in level data")
 
@@ -818,6 +952,26 @@ class PlayArchetype(_BaseArchetype):
             case _:
                 raise RuntimeError("Result is only accessible from the entity itself")
 
+    @classmethod
+    def update_life(
+        cls,
+        perfect_increment: int | None = None,
+        great_increment: int | None = None,
+        good_increment: int | None = None,
+        miss_increment: int | None = None,
+    ):
+        """Update the life of this archetype.
+
+        Values default to 0.
+
+        Args:
+            perfect_increment: The increment for perfect life.
+            great_increment: The increment for great life.
+            good_increment: The increment for good life.
+            miss_increment: The increment for miss life.
+        """
+        archetype_life_of(cls).update(perfect_increment, great_increment, good_increment, miss_increment)
+
 
 class WatchArchetype(_BaseArchetype):
     """Base class for watch mode archetypes.
@@ -863,13 +1017,14 @@ class WatchArchetype(_BaseArchetype):
         Runs first each frame.
 
         This is where logic affecting shared memory should be placed.
-        Other logic should be placed in `update_parallel` for better performance.
+        Other logic should typically be placed in
+        [`update_parallel`][sonolus.script.archetype.PlayArchetype.update_parallel] for better performance.
         """
 
     def update_parallel(self):
         """Parallel update callback.
 
-        Runs after `touch` each frame.
+        Runs after [`touch`][sonolus.script.archetype.PlayArchetype.touch] each frame.
 
         This is where most gameplay logic should be placed.
         """
@@ -911,7 +1066,7 @@ class WatchArchetype(_BaseArchetype):
             raise RuntimeError("Calling life is only allowed within a callback")
         match self._data_:
             case _ArchetypeSelfData() | _ArchetypeReferenceData():
-                return _deref(ctx().blocks.ArchetypeLife, self.id() * ArchetypeLife._size_(), ArchetypeLife)
+                return _deref(ctx().blocks.ArchetypeLife, self.id * ArchetypeLife._size_(), ArchetypeLife)
             case _:
                 raise RuntimeError("Life is not available in level data")
 
@@ -935,6 +1090,26 @@ class WatchArchetype(_BaseArchetype):
         if cls._exported_fields_:
             raise RuntimeError("Watch archetypes cannot have exported fields")
 
+    @classmethod
+    def update_life(
+        cls,
+        perfect_increment: int | None = None,
+        great_increment: int | None = None,
+        good_increment: int | None = None,
+        miss_increment: int | None = None,
+    ):
+        """Update the life of this archetype.
+
+        Values default to 0.
+
+        Args:
+            perfect_increment: The increment for perfect life.
+            great_increment: The increment for great life.
+            good_increment: The increment for good life.
+            miss_increment: The increment for miss life.
+        """
+        archetype_life_of(cls).update(perfect_increment, great_increment, good_increment, miss_increment)
+
 
 class PreviewArchetype(_BaseArchetype):
     """Base class for preview mode archetypes.
@@ -1023,7 +1198,7 @@ def archetype_life_of(archetype: type[_BaseArchetype] | _BaseArchetype) -> Arche
         raise RuntimeError("Calling archetype_life_of is only allowed within a callback")
     match ctx().global_state.mode:
         case Mode.PLAY | Mode.WATCH:
-            return _deref(ctx().blocks.ArchetypeLife, archetype.id() * ArchetypeLife._size_(), ArchetypeLife)
+            return _deref(ctx().blocks.ArchetypeLife, archetype.id * ArchetypeLife._size_(), ArchetypeLife)
         case _:
             raise RuntimeError(f"Archetype life is not available in mode '{ctx().global_state.mode}'")
 
@@ -1094,7 +1269,7 @@ class WatchEntityInput(Record):
 class EntityRef[A: _BaseArchetype](Record):
     """Reference to another entity.
 
-    May be used with `Any` to reference an unknown archetype.
+    May be used with `typing.Any` to reference an unknown archetype.
 
     Usage:
         ```python
@@ -1175,3 +1350,7 @@ class StandardArchetypeName(StrEnum):
 
     TIMESCALE_CHANGE = "#TIMESCALE_CHANGE"
     """Timescale change marker"""
+
+
+type AnyArchetype = PlayArchetype | WatchArchetype | PreviewArchetype
+"""Union of all archetype types."""
