@@ -33,7 +33,7 @@ class Box[T](Record):
         x: T = ...
         y: T = ...
         box = Box(x)
-        box.value = y  # Works regardless of whether x is a Num or not
+        box.value = y  # Works regardless of whether x is a Num, array, or record
         ```
     """
 
@@ -337,7 +337,7 @@ class ArrayPointer[T](Record, ArrayLike[T]):
             raise TypeError("ArrayPointer values cannot be accessed outside of a context")
         return _deref(
             # Allows a compile time constant block so we can warn based on callback read/write access
-            (self._value["block"]._is_py_() and self._value["block"]._as_py_()) or self.block,
+            (self._value_["block"]._is_py_() and self._value_["block"]._as_py_()) or self.block,
             self.offset + Num._accept_(item) * Num._accept_(self.element_type()._size_()),
             self.element_type(),
         )
